@@ -7,6 +7,10 @@ import numpy as np
 import numba as nb
 from cddm.conf import C,F, I64, NUMBA_TARGET, NUMBA_FASTMATH, NUMBA_CACHE
 from cddm.fft import _fft, _ifft
+from cddm.decorators import doc_inherit
+
+
+
 
 #Some useful functions
 
@@ -22,7 +26,7 @@ def mean(a,b):
 
 @nb.vectorize([F(F,F),C(C,C)], target = NUMBA_TARGET, cache = NUMBA_CACHE, fastmath = NUMBA_FASTMATH)
 def choose(a,b):
-    """Chooses data, randomly"""
+    """Chooses data randomly"""
     r = np.random.rand()
     if r >= 0.5:
         return a
@@ -33,11 +37,142 @@ def choose(a,b):
 def convolve(a, out):
     """Convolves input array with kernel [0.25,0.5,0.25]"""
     n = len(out)
-    assert n > 2
-    out[0] = a[0] #at the end we do not use this boundary data so it does not matter what we set here.
-    out[n-1] = a[n-1]    
+    assert n > 2  
+    result = a[0]
     for i in range(1,n-1):
-        out[i] = 0.25*(a[i-1]+2*a[i]+a[i+1])
+        out[i-1] = result
+        result = 0.25*(a[i-1]+2*a[i]+a[i+1])
+
+    out[i] = result
+    out[-1] = a[-1]
+
+@nb.guvectorize([(F[:],F[:],F[:],F[:])],"(n),(m),(m)->(n)", target = NUMBA_TARGET, cache = NUMBA_CACHE, fastmath = NUMBA_FASTMATH)
+def interpolate(x_new, x,y,out):
+    """Linear interpolation"""
+    assert len(x) >= 2
+    for i in range(len(x_new)):
+        xi = x_new[i]
+        for j in range(1,len(x)):
+            x0 = x[j-1]
+            x1 = x[j]
+            if xi <= x1:
+                #interpolate or extrapolate backward
+                deltay = y[j] - y[j-1]
+                deltax = x1 - x0
+                out[i] = (xi - x0) * deltay/ deltax +  y[j-1]
+                break
+        #extrapolate forward
+        if xi > x1:
+            deltay = y[-1] - y[-2]
+            deltax = x1 - x0
+            out[i] = (xi - x0) * deltay/deltax +  y[-2]            
+
+@nb.guvectorize([(F[:],F[:],F[:],F[:])],"(n), (m),(m)->(n)", target = NUMBA_TARGET, cache = NUMBA_CACHE, fastmath = NUMBA_FASTMATH)
+def _log_interpolate(x_new, x,y, out):
+    """Linear interpolation in semilogx space."""
+    assert len(x) >= 2
+    for i in range(len(x_new)):
+        xi = x_new[i]
+        log = False
+        if xi > 1:
+            xi = np.log(xi)
+            log = True
+        for j in range(1,len(x)):
+            x0 = x[j-1]
+            x1 = x[j]            
+            if x0 >= 1 and log == True:
+                x0 = np.log(x0)
+                x1 = np.log(x1)
+            if x_new[i] <= x[j]:
+                #interpolate or extrapolate backward
+                deltay = y[j] - y[j-1]
+                deltax = x1-x0
+                out[i] = (xi - x0) * deltay / deltax +  y[j-1]
+                break
+        #extrapolate forward for data points outside of the domain
+        if xi > x1:            
+            deltay = y[-1] - y[-2]
+            deltax = x1 - x0
+            out[i] = (xi - x0) * deltay/deltax +  y[-2]    
+
+def log_interpolate(x_new, x,y, out = None):
+    """Linear interpolation in semilogx space."""
+    #wrapped to suprres divide by zero warning numba issuu #4793
+    with np.errstate(divide='ignore'):
+        return _log_interpolate(x_new, x,y, out)
+    
+log_interpolate.__doc__ = _log_interpolate.__doc__
+
+@nb.guvectorize([(F[:],F[:])],"(n)->(n)", target = NUMBA_TARGET, cache = NUMBA_CACHE, fastmath = NUMBA_FASTMATH)
+def median(array, out):
+    """Performs median filter."""
+    n = len(array)
+    assert n > 2
+    result_out = array[0]
+    out[-1] = array[-1]
+    for i in range(1,n-1):
+        if array[i] < array[i+1]:
+            if array[i] < array[i-1]:
+                result = min(array[i+1],array[i-1])
+            else:
+                result = array[i]
+        else:
+            if array[i] < array[i-1]:
+                result = array[i]
+            else:
+                result = max(array[i+1],array[i-1])
+        out[i-1] = result_out
+        result_out = result
+    out[i] = result_out
+    #out[n-1] = result_out
+    #out[0] = out[1]
+
+@nb.vectorize([F(F,F,F)], target = NUMBA_TARGET, cache = NUMBA_CACHE, fastmath = NUMBA_FASTMATH)    
+def weighted_sum(x, y, weight):
+    """Performs weighted sum of two data sets, given the weight data.
+    Weight must be normalized between 0 and 1. Performs:
+    `x * weight + (1.- weight) * y`
+    """ 
+    return x * weight + (1.- weight) * y
+    
+@nb.guvectorize([(F[:],F[:])],"(n)->(n)", target = NUMBA_TARGET, cache = NUMBA_CACHE, fastmath = NUMBA_FASTMATH)
+def _median_slow(array, out):
+    """Performs median filter. slow implementation... for testing"""
+    n = len(array)
+    assert n > 2
+    for i in range(1,n-1):
+        median = np.sort(array[i-1:i+2])[1]
+        out[i] = median
+    out[0] = array[0]
+    out[-1] = array[-1]
+    #out[0] = out[1]
+    #out[n-1] = out[n-2]
+
+@nb.guvectorize([(F[:],F[:])],"(n)->(n)", target = NUMBA_TARGET, cache = NUMBA_CACHE, fastmath = NUMBA_FASTMATH)
+def decreasing(array, out):
+    """Performs decreasing filter. Each next element must be smaller or equal"""
+    n = len(array)
+    for i in range(n):
+        if i == 0:
+            out[0] = array[0]
+        else:
+            if array[i] < out[i-1]:
+                out[i] = array[i]
+            else:
+                out[i] = out[i-1]
+                
+@nb.guvectorize([(F[:],F[:])],"(n)->(n)", target = "cpu", cache = NUMBA_CACHE, fastmath = NUMBA_FASTMATH)
+def increasing(array,out):
+    """Performs increasing filter. Each next element must be greater or equal"""
+    n = len(array)
+    for i in range(1,n):
+        if i == 0:
+            out[0] = array[0]
+        else:
+            if array[i] > out[i-1]:
+                out[i] = array[i]
+            else:
+                out[i] = out[i-1]
 
 #------------------------------------------------
 # low level numba-optimized computation functions
@@ -418,4 +553,20 @@ def _normalize_ccorr_1(data, count, bg1, bg2, sq):
     
     return tmp + (0.5 * d2)
 
+@nb.vectorize([F(F)],target = NUMBA_TARGET, cache = NUMBA_CACHE, fastmath = NUMBA_FASTMATH)
+def weight_from_g1(g1):
+    """Computes weight for weighted normalization from normalized and scaled 
+    correlation function"""
+    tmp1 = (g1 - 1)**2
+    tmp2 = g1**2 + 1
+    return tmp1/tmp2
+    
+@nb.vectorize([F(F)],target = NUMBA_TARGET, cache = NUMBA_CACHE, fastmath = NUMBA_FASTMATH)
+def weight_from_d(d):
+    """Computes weight for weighted normalization from normalized and scaled 
+    image structure function"""
+    g1 = 1 - d/2.
+    tmp1 = (g1 - 1)**2
+    tmp2 = g1**2 + 1
+    return tmp1/tmp2
 
