@@ -38,11 +38,13 @@ from __future__ import absolute_import, print_function, division
 
 import numpy as np
 from cddm.conf import CDTYPE, FDTYPE, IDTYPE,I64DTYPE
-from cddm.print_tools import print1,print2, print_frame_rate
+from cddm.print_tools import print1,print2, print_frame_rate, enable_prints, disable_prints
 import time
 from functools import reduce 
 from cddm.fft import _fft
-from cddm._core_nb import _cross_corr_fft_regular, _cross_corr_fft, \
+from cddm.avg import weight_from_data, weighted_sum
+from cddm._core_nb import log_interpolate,\
+   _cross_corr_fft_regular, _cross_corr_fft, \
     _auto_corr_fft_regular,_auto_corr_fft,\
   _cross_corr_regular, _cross_corr, _cross_corr_vec, \
   _cross_diff_regular,_cross_diff,_cross_diff_vec, \
@@ -55,13 +57,49 @@ from cddm._core_nb import _cross_corr_fft_regular, _cross_corr_fft, \
   _normalize_ccorr_0,_normalize_ccorr_1,_normalize_ccorr_2,_normalize_ccorr_3
 
 NORM_BASELINE = 0
-"""baseline normalization"""
+"""baseline normalization flag"""
 
 NORM_COMPENSATED = 1
-"""compensated normalization (cross-diff)"""
+"""compensated normalization (cross-diff) flag"""
 
 NORM_SUBTRACTED = 2
-"""background subtraction normalization"""
+"""background subtraction normalization flag"""
+
+NORM_WEIGHTED = 4
+"""weighted normalization flag"""
+
+def norm_flags(compensated = False, subtracted = False, weighted = False):
+    """Return normalization flags from the parameters.
+    
+    Parameters
+    ----------
+    compensated : bool
+        Whether to set COMPENSATED normalization flag
+    subtracted : bool
+        Whether to set SUBTRACTED normalization flag
+    weighted : bool
+        Whether to set WEIGHTED normalization flag
+    
+    Returns
+    -------
+    norm : int
+        Normalization flagse
+        
+    Examples
+    --------
+    >>> norm_flags(True, False, True)
+    5
+    >>> norm_flags(True, True, False)
+    3
+    """
+    norm = NORM_BASELINE
+    if compensated == True:
+        norm = norm | NORM_COMPENSATED
+    if subtracted == True:
+        norm = norm | NORM_SUBTRACTED
+    if weighted == True:
+        norm = norm | NORM_WEIGHTED
+    return norm
 
 #--------------------------
 #Core correlation functions
@@ -974,7 +1012,7 @@ def _default_norm(norm, method, cross = True):
     if method == "diff":
         supported = (1,3) if cross else (1,)
     else:
-        supported = (0,1,2,3)
+        supported = (0,1,2,3,4,5,7,6) #so that 6 is default
     if norm is None:
         norm = supported[-1]
     if norm not in supported:
@@ -1072,7 +1110,7 @@ def acorr(f, t = None, fs = None,  n = None,  norm = None,
     else:
         _sum = auto_sum
         
-    if norm & NORM_COMPENSATED and correlate:
+    if (norm & NORM_COMPENSATED or norm & NORM_WEIGHTED) and correlate:
         if fs is None:
             fs =  abs2(f)
         else:
@@ -1217,7 +1255,7 @@ def ccorr(f1,f2,t1 = None, t2 = None,  n = None,
         _sum = cross_sum
 
         
-    if norm & NORM_COMPENSATED and correlate:
+    if (norm & NORM_COMPENSATED or norm & NORM_WEIGHTED) and correlate:
         print2("... tau sum square")
         if f1s is None:
             f1s =  abs2(f1)
@@ -1432,19 +1470,23 @@ def _norm_from_ccorr_data(data, norm = None):
     if s1 is None:
         if sq is None:
             default_norm = 0
+            available_norm = 0
         else:
-            default_norm = 1
+            default_norm = 4
+            available_norm = 5
     else:
         if sq is None:
             default_norm = 2
+            available_norm = 2
         else:
-            default_norm = 3 
+            default_norm = 6
+            available_norm = 7
     if norm is None:
         return default_norm
     else:
-        if norm not in (0,1,2,3):
-            raise ValueError("Normalization mode can be 0,1,2 or 3")
-        if  (norm & default_norm) == norm:
+        if norm not in range(8):
+            raise ValueError("Normalization mode must be in range(8)")
+        if  (norm & available_norm) == norm:
             return norm
         else:
             raise ValueError("Normalization mode incompatible with input data.")
@@ -1490,18 +1532,39 @@ def _corr2diff(data, variance, mask):
     return data
 
 def _variance2offset(variance, mask):
+    #offset for norm == 2 type of normalization
+    return scale_factor(variance, mask)[..., None]
+    
+def scale_factor(variance, mask = None):
+    """Computes the normalization scaling factor from the variance data.
+    
+    You can divide the computed correlation data with this factor to normalize
+    data between (0,1) for correlation mode, or (0,2) for difference mode.
+    
+    Parameters
+    ----------
+    variance : (ndarray, ndarray) or ndarray
+        A variance data (as returned from :func:`.stats`)
+    mask : ndarray
+        A boolean mask array, if computation was performed on masked data,
+        this applys mask to the variance data.
+        
+    Returns
+    -------
+    scale : ndarray
+        A scaling factor for normalization
+    """
     if variance is None:
         raise ValueError("You must provide variance data for normalization")
-    #offset for norm == 2 type of normalization
     try:
         v1, v2 = variance
-        offset = 0.5*np.asarray((v1 + v2))
+        scale = 0.5*np.asarray((v1 + v2))
     except:
-        offset = np.asarray(variance)  
+        scale = np.asarray(variance)  
     if mask is None:
-        return offset[..., None]
+        return scale
     else:
-        return offset[mask, None]
+        return scale[mask]
 
 def _inspect_background(background, norm, mask):
     if background is not None:
@@ -1616,9 +1679,13 @@ def normalize(data, background = None, variance = None, norm = None,  mode = "co
     #determine default normalization if not specified by user
     norm = _default_norm_from_data(data, method, norm)
 
+#    if len(data[1].shape) > 1 and norm & NORM_WEIGHTED:
+#        #if multilevel data, subtracted or baseline is best, so force it here.
+#        norm = norm & NORM_SUBTRACTED
+
     # scale factor for normalization 
     if scale == True:
-        _scale_factor = _variance2offset(variance,mask)
+        _scale_factor = scale_factor(variance,mask)
     
     print2("   * background : {}".format(background is not None))
     print2("   * variance   : {}".format(variance is not None))
@@ -1626,7 +1693,38 @@ def normalize(data, background = None, variance = None, norm = None,  mode = "co
     print2("   * scale      : {}".format(scale))
     print2("   * mode       : {}".format(mode))
     print2("   * mask       : {}".format(mask is not None))
-
+    
+    if (norm & NORM_WEIGHTED):
+        level = disable_prints()
+        norm_comp = (norm & NORM_SUBTRACTED)|NORM_COMPENSATED
+        comp_data = normalize(data, background, variance, norm = norm_comp,  mode = mode, 
+              scale = scale, mask = mask)
+        norm_base = norm & NORM_SUBTRACTED
+        base_data = normalize(data, background, variance, norm = norm_base,  mode = mode, 
+              scale = scale, mask = mask)
+        
+        _multilevel = True if len(data[1].shape) > 1 else False
+        
+        x_avg,y_avg = _data_estimator(comp_data, size = 8, n = 3, multilevel = _multilevel)
+        
+        _scale_factor = 1. if scale == True else scale_factor(variance,mask)
+        
+        if _multilevel:
+            shape = np.array(comp_data.shape)
+            shape[1:-1] = 1
+            _x_interp = np.arange(comp_data.shape[-1])
+            x_interp = np.empty(shape = shape, dtype = int)
+            for x_level in x_interp:
+                x_level[...] = _x_interp
+                _x_interp *= 2
+        else:
+            x_interp = np.arange(data[1].shape[-1])
+        
+        weight = weight_from_data(y_avg, scale_factor = _scale_factor, mode = mode)
+        weight = log_interpolate(x_interp,x_avg, weight)
+        enable_prints(level) 
+        return weighted_sum(base_data,comp_data, weight)
+        
     count = data[1]
     
     if mask is not None:
@@ -1685,9 +1783,83 @@ def normalize(data, background = None, variance = None, norm = None,  mode = "co
             if mode == "corr":
                 result = _diff2corr(result, variance, mask)
     if scale == True:
-        result /= _scale_factor
+        result /= _scale_factor[...,None]
     return result
 
+#def weight_from_data(base_data, comp_data,  scale_factor = 1., mode = "corr", norm = NORM_BASELINE, multitau = False):
+#    comp_data_avg = average(comp_data)
+#    base_data_avg = average(base_data)
+#
+#    comp_data_avg[...,0] =  scale_factor if mode == "corr" else 0.
+#    median(comp_data_avg,comp_data_avg)
+#    comp_data_avg[...,0] =  scale_factor if mode == "corr" else 0.
+#    median(comp_data_avg,comp_data_avg)
+#    comp_data_avg[...,0] =  scale_factor if mode == "corr" else 0.
+#            
+#    weight = base_weight(comp_data_avg, scale_factor = scale_factor, mode = mode)
+#    
+#    #mask =  weight > 0.5
+#    #comp_data_avg[mask] =  base_data_avg[mask]
+#    #median(comp_data_avg,comp_data_avg)
+#    
+#    #weight = base_weight(comp_data_avg, scale_factor = scale_factor, mode = mode)
+#
+#    if norm & NORM_COMPENSATED:
+#        #avg_data = weighted_sum(base_data_avg, comp_data_avg, weight)
+#        weight = comp_weight(comp_data_avg, scale_factor = scale_factor, mode = mode)
+#        #weight = comp_weight2(base_data, comp_data , comp_data_avg)  
+#
+#    if multitau == True:
+#        n = weight.shape[-1]
+#        # make sure weights are increasing.
+#        for i,w in enumerate(weight):
+#            if i > 0:
+#                #make first weight equal last weight from previous level
+#                weight[i,...,n//2] = weight[i-1,...,-1]
+#                #this way it can only increase when going through level.
+#            increasing(w,out = w)   
+#    return weight
+
+
+def _data_estimator(data, size = 8, n = 3, multilevel = False):
+    from cddm.multitau import log_average, merge_multilevel
+    from cddm.avg import denoise
+    if multilevel == False:
+        x,y = log_average(data,size)
+    else:
+        x,y = merge_multilevel(data)
+    return x, denoise(y, n = n, out = y)
+
+
+#def average(x, size = 2):
+#    """Performs averaging of normalized linear-spaced data with delay-dependent
+#    kernel. 
+#    
+#    You must first normalize with :func:`.core.normalize` before averaging!
+#    
+#    Parameters
+#    ----------
+#    data : array
+#        Input array of linear-spaced data
+#    size : int
+#        Sampling size. Number of data points per each doubling of time.
+#        Any positive number is valid.
+#        
+#    Returns
+#    -------
+#    avg : ndarray
+#       Averaged data of same shape as the original data.
+#    """
+#        
+#    from cddm.multitau import log_average
+#    x = np.asarray(x)
+#    t,xl= log_average(x,size)
+#    new_t = np.arange(x.shape[-1])
+#    out = log_interpolate(new_t,t,xl)
+#    return out
+
+
+
 #if __name__ == "__main__":
 #    import doctest
 #    doctest.testmod()
