@@ -25,31 +25,36 @@ For time averaging of linear spaced data use:
 
 from __future__ import absolute_import, print_function, division
 
-from cddm.core import normalize, ccorr,acorr, NORM_COMPENSATED, NORM_SUBTRACTED
+from cddm.core import normalize, ccorr,acorr, NORM_COMPENSATED, NORM_SUBTRACTED, NORM_WEIGHTED
 from cddm.core import _transpose_data, _inspect_cross_arguments,_default_norm,\
         _is_aligned, _move_axis_and_align, _inspect_auto_arguments, abs2,\
         reshape_input, reshape_output, thread_frame_shape, reshape_frame
 import numpy as np
-import numba as nb
-from cddm.conf import CDTYPE, FDTYPE, IDTYPE, C,F, NUMBA_TARGET, NUMBA_FASTMATH, NUMBA_CACHE
+
+from cddm.conf import CDTYPE, FDTYPE, IDTYPE
 from cddm.print_tools import print_progress,  print_frame_rate, enable_prints, disable_prints
 from cddm.print_tools import print1, print2
 
-from cddm._core_nb import mean, choose, convolve, _add_stats_vec, _calc_stats_vec
+from cddm._core_nb import mean, choose, convolve, _calc_stats_vec
+from cddm.core import _method_from_data,_inspect_scale,_inspect_mode,_default_norm_from_data, scale_factor
+from cddm.avg import weighted_sum,weight_from_data, log_interpolate
 
 import time
 
 #: placehold for matplotlib plots so that they are not garbage collected diring live view
 _VIEWERS = {}
 
-BINNING_SLICE = 0
+BINNING_FIRST = 0
 """No binning (select every second element)"""
 BINNING_MEAN = 1
 """Binning (take mean value)"""
 BINNING_CHOOSE = 2
 """Binning with random selection"""
 
-def mean_data(data, axis = 0, out_axis = None):
+#for backward compatibility
+BINNING_SLICE = BINNING_FIRST
+
+def mean_data(data, axis = 0, out_axis = None,**kw):
     """Binning function. Takes data and performs channel binning over a specifed
     axis. If specified, also moves axis to out_axis."""
     if out_axis is None:
@@ -61,19 +66,26 @@ def mean_data(data, axis = 0, out_axis = None):
     out = np.empty(f1.shape, f1.dtype)
     return mean (f1, f2, out = out)
 
-def choose_data(data, axis = 0, out_axis = None):
+def choose_data(data, axis = 0, out_axis = None, r = None):
     """Instead of binning, this randomly selects data.
     If specified, also moves axis to out_axis."""
     if out_axis is None:
         out_axis = axis
+    if r is None:
+        r = np.random.rand()
     f = np.moveaxis(data,axis,-1)
     n = f.shape[-1] #this makes it possible to bin also odd-sized data.. skip one element
     f1 = np.moveaxis(f[...,0:n-1:2],-1,out_axis)
     f2 = np.moveaxis(f[...,1:n:2],-1,out_axis)
     out = np.empty(f1.shape, f1.dtype)
-    return  choose(f1,f2, out)
+    if r < 0.5:
+        out[...] = f2
+    else:
+        out[...] = f1
+    return out
+    #return  choose(f1,f2, out)
 
-def slice_data(data, axis = 0, out_axis = None):
+def slice_data(data, axis = 0, out_axis = None,**kw):
     """Slices data so that it takes every second channel. If specified, also moves axis to out_axis."""
     if out_axis is None:
         out_axis = axis
@@ -90,13 +102,13 @@ def _default_method_and_binning(method,binning):
             binning =  BINNING_MEAN
     elif method  == "diff":
         if binning is None:
-            binning =  BINNING_SLICE
+            binning =  BINNING_FIRST
     else:
         raise ValueError("Unknown method, must be 'corr', 'diff' or 'fft'")
     if method == "diff":
-        supported = (BINNING_CHOOSE,BINNING_SLICE)
+        supported = (BINNING_CHOOSE,BINNING_FIRST)
     else:
-        supported = (BINNING_MEAN,BINNING_SLICE, BINNING_CHOOSE)
+        supported = (BINNING_MEAN,BINNING_FIRST, BINNING_CHOOSE)
 
     if binning not in supported:
         raise ValueError("Binning mode {} not supported for method '{}'".format(binning,method))
@@ -106,7 +118,7 @@ def _get_binning_function(binning):
     """returns binning function"""
     if binning == BINNING_MEAN:
         _bin = mean_data
-    elif binning == BINNING_SLICE:
+    elif binning == BINNING_FIRST:
         _bin = slice_data   
     elif binning == BINNING_CHOOSE:
         _bin = choose_data
@@ -168,8 +180,8 @@ def _compute_multi(f1,f2 = None, t1 = None, t2 = None, axis = 0, period = 1, lev
     
     if correlate == True:
         #we need to track square of the signal
-        f1s = abs2(f1) if norm & NORM_COMPENSATED else None
-        f2s = abs2(f2) if norm & NORM_COMPENSATED and cross else None
+        f1s = abs2(f1) if (norm & NORM_COMPENSATED or norm & NORM_WEIGHTED) else None
+        f2s = abs2(f2) if (norm & NORM_COMPENSATED or norm & NORM_WEIGHTED) and cross else None
     else:
         f1s,f2s = None,None
 
@@ -210,7 +222,7 @@ def _compute_multi(f1,f2 = None, t1 = None, t2 = None, axis = 0, period = 1, lev
     count_slow = np.zeros((nlevel, n_slow,),IDTYPE)
     m1, m2, sq = None, None, None
  
-    if norm & NORM_COMPENSATED and correlate == True:
+    if (norm & NORM_COMPENSATED or norm & NORM_WEIGHTED) and correlate == True:
         sq = np.zeros(slow_shape, FDTYPE) 
         sq = _transpose_data(sq, new_axis) 
         
@@ -239,7 +251,7 @@ def _compute_multi(f1,f2 = None, t1 = None, t2 = None, axis = 0, period = 1, lev
     for i in range(nlevel):
         f1 = _bin(f1,axis, new_axis)
         f2 = _bin(f2,axis, new_axis) if cross else None
-        if norm & NORM_COMPENSATED and correlate:
+        if (norm & NORM_COMPENSATED or norm & NORM_WEIGHTED) and correlate:
             f1s = _bin(f1s,axis, new_axis)
             f2s = _bin(f2s,axis, new_axis) if cross else None
         axis = new_axis
@@ -367,7 +379,7 @@ def acorr_multi(f, t = None,  level_size = 2**4, norm = None, method = None, ali
     return _compute_multi(f, t1 = t, axis = axis, period = period, level_size = level_size, align = align,
                          binning = binning,  nlevel = nlevel, method = method, norm = norm,thread_divisor = thread_divisor, mask = mask)
     
-def _add_data1(i,x, out, binning = True):
+def _add_data1(i,x, out, binning = True, r = None):
     chunk_size = out.shape[-2]
     n_decades = out.shape[0]
     
@@ -382,7 +394,11 @@ def _add_data1(i,x, out, binning = True):
             if binning == BINNING_MEAN:
                 x = mean(x, out[j-1,...,index_top-1,:], out[j,...,index_low,:])
             elif binning == BINNING_CHOOSE:
-                x = choose(x, out[j-1,...,index_top-1,:], out[j,...,index_low,:])
+                if r is None:
+                    r = np.randomd.rand()
+                x = x if r > 0.5 else out[j-1,...,index_top-1,:]
+                out[j,...,index_low,:] = x
+                #x = choose(x, out[j-1,...,index_top-1,:], out[j,...,index_low,:])
             else:
                 out[j,...,index_low,:] = x
             
@@ -390,7 +406,7 @@ def _add_data1(i,x, out, binning = True):
         else:
             break
 
-def _add_data2(i,x1, x2, out1, out2, binning = True):
+def _add_data2(i,x1, x2, out1, out2, binning = True, r = None):
     chunk_size = out1.shape[-2]
     n_decades = out1.shape[0]
     
@@ -407,8 +423,14 @@ def _add_data2(i,x1, x2, out1, out2, binning = True):
                 x1 = mean(x1, out1[j-1,...,index_top-1,:], out1[j,...,index_low,:])
                 x2 = mean(x2, out2[j-1,...,index_top-1,:], out2[j,...,index_low,:])  
             elif binning == BINNING_CHOOSE:
-                x1 = choose(x1, out1[j-1,...,index_top-1,:], out1[j,...,index_low,:])
-                x2 = choose(x2, out2[j-1,...,index_top-1,:], out2[j,...,index_low,:])  
+                if r is None:
+                    r = np.randomd.rand()
+                x1 = x1 if r > 0.5 else out1[j-1,...,index_top-1,:]
+                x2 = x2 if r > 0.5 else out2[j-1,...,index_top-1,:]
+                out1[j,...,index_low,:] = x1
+                out2[j,...,index_low,:] = x2
+                #x1 = choose(x1, out1[j-1,...,index_top-1,:], out1[j,...,index_low,:])
+                #x2 = choose(x2, out2[j-1,...,index_top-1,:], out2[j,...,index_low,:])  
             else:
                 out1[j,...,index_low,:] = x1
                 out2[j,...,index_low,:] = x2
@@ -567,7 +589,7 @@ def _compute_multi_iter(data, t1, t2 = None, period = 1, level_size = 16,
             else:
                 m1_fast, m2_fast = None, None
                 
-            if norm & NORM_COMPENSATED and correlate == True:
+            if (norm & NORM_COMPENSATED or norm & NORM_WEIGHTED) and correlate == True:
                 sq_fast = np.zeros(fast_shape, FDTYPE)
                 sq_fast = _transpose_data(sq_fast) 
             else:
@@ -593,7 +615,7 @@ def _compute_multi_iter(data, t1, t2 = None, period = 1, level_size = 16,
                 m2_slow = _transpose_data(m2_slow) if cross  == True else None
             else:
                 m1_slow, m2_slow = None, None
-            if norm & NORM_COMPENSATED and correlate == True:
+            if (norm & NORM_COMPENSATED or norm & NORM_WEIGHTED) and correlate == True:
                 sq_slow = np.zeros(slow_shape, FDTYPE)  
                 sq_slow = _transpose_data(sq_slow) 
             else:
@@ -615,7 +637,7 @@ def _compute_multi_iter(data, t1, t2 = None, period = 1, level_size = 16,
                 data_slow = out_slow, count_slow, m1_slow, m2_slow
                      
             
-            if norm & NORM_COMPENSATED and correlate == True:
+            if (norm & NORM_COMPENSATED or norm & NORM_WEIGHTED) and correlate == True:
                 #allocate memory for square data
                 sq1 = np.empty(data_shape, FDTYPE)
                 sq2 = np.empty(data_shape, FDTYPE) if cross  == True else None
@@ -648,30 +670,32 @@ def _compute_multi_iter(data, t1, t2 = None, period = 1, level_size = 16,
              
                 
                 for i in range(half_chunk_size):
+                    r = np.random.rand()
                     x1 = fdata1[0,...,i,:] - bg1
                     x2 = fdata2[0,...,i,:] - bg2 if cross == True else None
                     if cross == True:
-                        _add_data2(i,x1, x2, fdata1,fdata2, binning)
-                        if norm & NORM_COMPENSATED and correlate:
-                            _add_data2(i,abs2(x1), abs2(x2), sq1,sq2, binning) 
+                        _add_data2(i,x1, x2, fdata1,fdata2, binning, r)
+                        if (norm & NORM_COMPENSATED or norm & NORM_WEIGHTED) and correlate:
+                            _add_data2(i,abs2(x1), abs2(x2), sq1,sq2, binning, r) 
                     else:
-                        _add_data1(i,x1, fdata1, binning)
-                        if norm & NORM_COMPENSATED and correlate:
-                            _add_data1(i,abs2(x1),sq1,binning)
+                        _add_data1(i,x1, fdata1, binning,r)
+                        if (norm & NORM_COMPENSATED or norm & NORM_WEIGHTED) and correlate:
+                            _add_data1(i,abs2(x1),sq1,binning,r)
         else:
             if bg1 is not None:
                 x1 = x1 - bg1
             if bg2 is not None and cross:
                 x2 = x2 - bg2
                 
+            r = np.random.rand() 
             if cross == True:
-                _add_data2(i,x1, x2, fdata1,fdata2, binning)
-                if norm & NORM_COMPENSATED and correlate:
-                    _add_data2(i,abs2(x1), abs2(x2), sq1,sq2, binning)
+                _add_data2(i,x1, x2, fdata1,fdata2, binning,r)
+                if (norm & NORM_COMPENSATED or norm & NORM_WEIGHTED) and correlate:
+                    _add_data2(i,abs2(x1), abs2(x2), sq1,sq2, binning,r)
             else:
-                _add_data1(i,x1, fdata1,binning)
-                if norm & NORM_COMPENSATED and correlate:
-                    _add_data1(i,abs2(x1),sq1,binning)                
+                _add_data1(i,x1, fdata1,binning,r)
+                if (norm & NORM_COMPENSATED or norm & NORM_WEIGHTED) and correlate:
+                    _add_data1(i,abs2(x1),sq1,binning,r)                
                 
         if i % (half_chunk_size) == half_chunk_size -1: 
 
@@ -704,8 +728,8 @@ def _compute_multi_iter(data, t1, t2 = None, period = 1, level_size = 16,
                 np.subtract(out_var1, abs2(out_bg1), out_var1)
                 np.subtract(out_var2, abs2(out_bg2), out_var2)  if cross else None
                                
-            f1s = sq1[0,...,fstart1:fstop1,:] if norm & NORM_COMPENSATED and correlate else None
-            f2s = sq2[0,...,fstart1:fstop1,:] if norm & NORM_COMPENSATED and correlate and cross else None
+            f1s = sq1[0,...,fstart1:fstop1,:] if (norm & NORM_COMPENSATED or norm & NORM_WEIGHTED) and correlate else None
+            f2s = sq2[0,...,fstart1:fstop1,:] if (norm & NORM_COMPENSATED or norm & NORM_WEIGHTED) and correlate and cross else None
             
             out = data_fast
                 
@@ -718,13 +742,13 @@ def _compute_multi_iter(data, t1, t2 = None, period = 1, level_size = 16,
 
             if istart2 >= 0 and mode == "full":
                 
-                f1s = sq1[0][...,fstart1:fstop1,:] if norm & NORM_COMPENSATED and correlate else None
+                f1s = sq1[0][...,fstart1:fstop1,:] if (norm & NORM_COMPENSATED or norm & NORM_WEIGHTED) and correlate else None
                 
                 if cross:
-                    f2s = sq2[0][...,fstart2:fstop2,:] if norm & NORM_COMPENSATED and correlate else None
+                    f2s = sq2[0][...,fstart2:fstop2,:] if (norm & NORM_COMPENSATED or norm & NORM_WEIGHTED) and correlate else None
                     ccorr(fdata1[0][...,fstart1:fstop1,:],fdata2[0][...,fstart2:fstop2,:],t1[istart1:istop1],t2[istart2:istop2],f1s = f1s, f2s = f2s, axis = -2, n = n_fast,norm = norm,aout = out, method = method) 
                 else:
-                    f2s = sq1[0][...,fstart2:fstop2,:] if norm & NORM_COMPENSATED and correlate else None
+                    f2s = sq1[0][...,fstart2:fstop2,:] if (norm & NORM_COMPENSATED or norm & NORM_WEIGHTED) and correlate else None
                     if correlate:
                         out = data_fast[0],data_fast[1], data_fast[2], None, None 
                     else:
@@ -734,8 +758,8 @@ def _compute_multi_iter(data, t1, t2 = None, period = 1, level_size = 16,
                         m1_fast += out[3] /2
                         m1_fast += out[4] /2
                 if cross:
-                    f1s = sq1[0][...,fstart2:fstop2,:] if norm & NORM_COMPENSATED and correlate else None
-                    f2s = sq2[0][...,fstart1:fstop1,:] if norm & NORM_COMPENSATED and correlate else None  
+                    f1s = sq1[0][...,fstart2:fstop2,:] if (norm & NORM_COMPENSATED or norm & NORM_WEIGHTED) and correlate else None
+                    f2s = sq2[0][...,fstart1:fstop1,:] if (norm & NORM_COMPENSATED or norm & NORM_WEIGHTED) and correlate else None  
                     ccorr(fdata1[0][...,fstart2:fstop2,:],fdata2[0][...,fstart1:fstop1,:],t1[istart2:istop2],t2[istart1:istop1],f1s = f1s, f2s = f2s,axis = -2, n = n_fast,norm = norm,aout = out, method = method) 
   
 
@@ -759,8 +783,8 @@ def _compute_multi_iter(data, t1, t2 = None, period = 1, level_size = 16,
                     istop2 = istop1 - half_chunk_size  
                 
   
-                    f1s = sq1[j,...,fstart1:fstop1,:] if norm & NORM_COMPENSATED and correlate else None
-                    f2s = sq2[j,...,fstart1:fstop1,:] if norm & NORM_COMPENSATED and correlate and cross else None  
+                    f1s = sq1[j,...,fstart1:fstop1,:] if (norm & NORM_COMPENSATED or norm & NORM_WEIGHTED) and correlate else None
+                    f2s = sq2[j,...,fstart1:fstop1,:] if (norm & NORM_COMPENSATED or norm & NORM_WEIGHTED) and correlate and cross else None  
                     
                     out =  tuple(((d[j-1] if d is not None else None) for d in data_slow))
                     
@@ -770,13 +794,13 @@ def _compute_multi_iter(data, t1, t2 = None, period = 1, level_size = 16,
                         acorr(fdata1[j,...,fstart1:fstop1,:],fs = f1s, axis = -2, norm = norm, n = n_slow, aout = out, method = method)
         
                     if istart2 >= 0 and mode == "full":
-                        f1s = sq1[j,...,fstart1:fstop1,:] if norm & NORM_COMPENSATED and correlate else None
+                        f1s = sq1[j,...,fstart1:fstop1,:] if (norm & NORM_COMPENSATED or norm & NORM_WEIGHTED) and correlate else None
                         
                         if cross:
-                            f2s = sq2[j,...,fstart2:fstop2,:] if norm & NORM_COMPENSATED and correlate else None
+                            f2s = sq2[j,...,fstart2:fstop2,:] if (norm & NORM_COMPENSATED or norm & NORM_WEIGHTED) and correlate else None
                             ccorr(fdata1[j,...,fstart1:fstop1,:],fdata2[j,...,fstart2:fstop2,:],t_slow[istart1:istop1],t_slow[istart2:istop2],f1s = f1s, f2s = f2s, axis = -2, n = n_fast,norm = norm,aout = out, method = method) 
                         else:
-                            f2s = sq1[j,...,fstart2:fstop2,:] if norm & NORM_COMPENSATED and correlate else None
+                            f2s = sq1[j,...,fstart2:fstop2,:] if (norm & NORM_COMPENSATED or norm & NORM_WEIGHTED) and correlate else None
                             _out =  tuple(((d[j-1] if d is not None else None) for d in data_slow))
                             if correlate:
                                 out = _out[0],_out[1], _out[2], None, None 
@@ -787,8 +811,8 @@ def _compute_multi_iter(data, t1, t2 = None, period = 1, level_size = 16,
                                 m1_slow[j-1] += out[3] /2
                                 m1_slow[j-1] += out[4] /2
                         if cross:
-                            f1s = sq1[j,...,fstart2:fstop2,:] if norm & NORM_COMPENSATED and correlate else None
-                            f2s = sq2[j,...,fstart1:fstop1,:] if norm & NORM_COMPENSATED and correlate else None  
+                            f1s = sq1[j,...,fstart2:fstop2,:] if (norm & NORM_COMPENSATED or norm & NORM_WEIGHTED) and correlate else None
+                            f2s = sq2[j,...,fstart1:fstop1,:] if (norm & NORM_COMPENSATED or norm & NORM_WEIGHTED) and correlate else None  
                             ccorr(fdata1[j,...,fstart2:fstop2,:],fdata2[j,...,fstart1:fstop1,:],t_slow[istart2:istop2],t_slow[istart1:istop1],f1s = f1s, f2s = f2s,axis = -2, n = n_fast,norm = norm,aout = out, method = method) 
 
                 else:
@@ -820,17 +844,18 @@ def _compute_multi_iter(data, t1, t2 = None, period = 1, level_size = 16,
     
     f1 = fdata1[-1]
     f2 = fdata2[-1] if cross else None
-    if norm & NORM_COMPENSATED and correlate:
+    if (norm & NORM_COMPENSATED or norm & NORM_WEIGHTED) and correlate:
         sq1 = sq1[-1]
         sq2 = sq2[-1] if cross else None
         
     _bin = _get_binning_function(binning)
          
     for j in range(max(n_decades,1),nlevel+1):
-        f1 = _bin(f1,-2)
-        f2 = _bin(f2,-2) if cross else None
-        sq1 = _bin(sq1,-2) if norm & NORM_COMPENSATED and correlate else None
-        sq2 = _bin(sq2,-2) if norm & NORM_COMPENSATED and correlate and cross else None
+        r = np.random.rand()
+        f1 = _bin(f1,-2, r = r)
+        f2 = _bin(f2,-2, r = r) if cross else None
+        sq1 = _bin(sq1,-2, r = r) if (norm & NORM_COMPENSATED or norm & NORM_WEIGHTED) and correlate else None
+        sq2 = _bin(sq2,-2, r = r) if (norm & NORM_COMPENSATED or norm & NORM_WEIGHTED) and correlate and cross else None
       
         out =  tuple(((d[j-1] if d is not None else None) for d in data_slow))
         
@@ -1019,7 +1044,157 @@ def iacorr_multi(data, t = None, level_size = 2**4, norm = 3, method = "corr", c
                     viewer.plot()
     return data
 
-def multilevel(data, level_size = 16):
+def triangular_kernel(n):
+    """Returns normalized triangular kernel for a given level integer
+    
+    Examples
+    --------
+    >>> triangular_kernel(0) #zero-th level
+    array([1.])
+    >>> triangular_kernel(1) #first level
+    array([0.25, 0.5 , 0.25])
+    """
+    size = 1
+    power = 2**n
+    for i in range(n):
+        size = size*2 +1
+    out = np.zeros(shape = (size,))
+    
+    for i in range(size//2):
+        out[i] = (i+1)/power/power
+        out[-i-1] = out[i]
+    out[size//2] = 1/power
+    return out
+
+def multilevel_count(count, level_size, binning = BINNING_MEAN):
+    """Returns effective number of measurements in multilevel data calculated
+    from linear time-space count data using :func:`multilevel`
+    
+    This function can be used to estimate the effective number of realizations
+    of each data point in multilevel data for error estimation.
+    
+    Parameters
+    ----------
+    count : ndarray
+        Count data,
+    level_size : int
+        Level size used in multilevel averaging.
+    binning : int
+        Binning mode that was used in :func:`multilevel`
+        
+    Returns
+    -------
+    x  : ndarray
+        Multilevel effective count array. Shape of this data depends on the 
+        length of the original data and the provided level_size parameter.
+    """
+    binning = int(binning)
+    #kernel = triangular_kernel(n)
+    #return 1/(kernel**2).sum()
+
+    size = int(level_size)
+    if size < 2:
+        raise ValueError("level_size must be greater than 1")
+    #determine multitau level (number of decades)
+    level = 0
+    while size * 2**(level) < count.shape[-1]:
+        level +=1
+    
+    shape = (level+1,size)
+    
+    out = np.empty(shape = shape, dtype = FDTYPE)
+    
+    count2 = count **2
+    
+    for l in range(level+1):
+        if binning == BINNING_MEAN:
+            kernel = triangular_kernel(l)
+        else:
+            kernel = np.array([1.]) #np.array([1./2**l])
+        kernel2 = kernel**2
+        
+        data = np.convolve(kernel2, count2, "same")
+        norm = np.convolve(kernel, count , "same")
+        data_sliced = data[0::2**l][0:size]
+        norm_sliced = norm[0::2**l][0:size]
+        n_sliced = data_sliced.shape[-1]
+        out[l,0:n_sliced] = norm_sliced**3 / data_sliced
+        
+        #missing data... fill nans
+        if n_sliced != size:
+            out[l,n_sliced:] = np.nan
+        
+    return out
+
+def ccorr_multi_count(count, period = 1, level_size = 16, binning = True):
+    """Returns an effective count data for the equivalent multilevel cross-correlation
+    calculation. 
+    
+    Parameters
+    ----------
+    count : int
+        Number of frames processed
+    period : int
+        The period argument used in correlation calculation
+    level_size : int
+        Level size used in correlation calculation
+    binning, int
+        Binning mode used in correlation calculation
+        
+    Returns
+    -------
+    cf,cs : ndarray, ndarray
+        Count arrays for the linear part and the multilevel part of the multilevel
+        correlation calculation.
+    """
+    count_fast = np.linspace(count/period*2, (count - level_size)/period*2 , period*level_size)
+    count_slow = np.linspace(count,1,count)*2
+    count_slow = multilevel_count(count_slow, level_size, binning)  
+    if binning != BINNING_MEAN:
+        n = count_slow.shape[0]
+        c = 2**(np.arange(n))
+        count_slow = count_slow/c[:,None]
+    
+    return count_fast, count_slow[1:]
+
+def acorr_multi_count(count, period = 1, level_size = 16, binning = True):
+    """Returns an effective count data for the equivalent multilevel auto-correlation
+    calculation. 
+    
+    Parameters
+    ----------
+    count : int
+        Number of frames processed
+    period : int
+        The period argument used in correlation calculation
+    level_size : int
+        Level size used in correlation calculation
+    binning, int
+        Binning mode used in correlation calculation
+        
+    Returns
+    -------
+    cf,cs : ndarray, ndarray
+        Count arrays for the linear part and the multilevel part of the multilevel
+        correlation calculation,  
+    """ 
+    
+    count_fast = np.linspace(count/period, (count - level_size)/period , period*level_size)
+    count_slow = np.linspace(count,1,count)
+    count_slow = multilevel_count(count_slow, level_size, binning)  
+    if binning != BINNING_MEAN:
+        n = count_slow.shape[0]
+        c = 2**(np.arange(n))
+        count_slow = count_slow/c[:,None]
+        
+    return count_fast, count_slow[1:]
+
+def _inspect_binning(binning):
+    if binning not in (BINNING_CHOOSE, BINNING_MEAN, BINNING_FIRST):
+        raise ValueError("Unsupported binning mode")
+    return binning
+    
+def multilevel(data, level_size = 16, binning = BINNING_MEAN):
     """Computes a multi-level version of the linear time-spaced data.
     
     Parameters
@@ -1028,13 +1203,16 @@ def multilevel(data, level_size = 16):
         Normalized correlation data 
     level_size : int
         Level size
+    binning : int
+        Binning mode, either BINNING_FIRST or BINNING_MEAN (default).
         
     Returns
     -------
     x  : ndarray
         Multilevel data array. Shape of this data depends on the length of the original
-        data and the provided parameter
+        data and the provided level_size parameter
     """
+    binning = _inspect_binning(binning)
     size = int(level_size)
     if size < 2:
         raise ValueError("level_size must be greater than 1")
@@ -1056,7 +1234,12 @@ def multilevel(data, level_size = 16):
         if n_sliced != size:
             out[l,...,n_sliced:] = np.nan
         if l != level:
-            data = convolve(data)[...,::2]
+            if binning == BINNING_MEAN:
+                data = convolve(data)[...,::2]
+            elif binning == BINNING_CHOOSE:
+                data = data[...,::2]
+            else:
+                data = choose_data(data, axis = -1)
     return out
 
 def merge_multilevel(data, mode = "full"):
@@ -1079,7 +1262,7 @@ def merge_multilevel(data, mode = "full"):
         x0 = np.arange(data.shape[-1])
         y0 = data[0,...]   
         x, y = merge_multilevel(data[1:], mode = "half")
-        return np.concatenate((x0,x),axis = -1), np.concatenate((y0,y),axis = -1)
+        return np.concatenate((x0,x*2),axis = -1), np.concatenate((y0,y),axis = -1)
     elif mode == "half":
         n = data.shape[-1]
         if n%2 != 0:
@@ -1089,7 +1272,7 @@ def merge_multilevel(data, mode = "full"):
         
         x_slow = np.empty((len(data),n), IDTYPE)
         for i,x in enumerate(x_slow):
-            x[:] = np.arange(n)*2**(i+1)
+            x[:] = np.arange(n)*2**(i)
         x = x_slow[:,n//2:].reshape((out_size,))
         data = data[...,n//2:]
         axes = [i for i in range(data.ndim)]
@@ -1099,7 +1282,7 @@ def merge_multilevel(data, mode = "full"):
         return x,y   
     else:
         raise ValueError("Unknown merging mode")
-        
+                 
 def log_average(data, size = 8):
     """Performs log average of normalized linear-spaced data.
     
@@ -1126,7 +1309,38 @@ def log_average(data, size = 8):
     ldata = multilevel(data, size*2)
     return merge_multilevel(ldata)
 
-def log_merge(lin,multi):
+def log_average_count(count, size = 8):
+    """Returns effective number of measurements in data calculated
+    from linear time-space count data using :func:`log_average`
+    
+    This function can be used to estimate the effective number of realizations
+    of each data point in multilevel data for error estimation.
+    
+    Parameters
+    ----------
+    count : ndarray
+        Count data, as returned by the `ccorr` or `acorr` functions
+    size : int
+        Sampling size that was used in :func:`log_average`
+        
+    Returns
+    -------
+    x, y : ndarray, ndarray
+        Time and log-spaced effetive count arrays.
+    """
+    data = multilevel_count(count, size * 2, binning = True)
+    x, y = merge_multilevel(data, mode = "full")
+    return x, y
+
+def _period_from_data(lin, multi):
+    nfast = lin.shape[-1]
+    nslow = multi.shape[-1]
+    period = nfast // nslow
+    if nfast % nslow != 0:
+        raise ValueError("Fast and slow data are not compatible")
+    return period
+
+def log_merge(lin,multi, binning = BINNING_MEAN):
     """Merges normalized multi-tau data.
     
     You must first normalize with :func:`normalize_multi` before merging!
@@ -1140,28 +1354,71 @@ def log_merge(lin,multi):
         Linear data
     multi : ndarray
         Multilevel data
+    binning : int
+        Binning mode used for multilevel calculation of the linear part of the data.
+        either BINNING_FIRST or BINNING_MEAN (default).
         
     Returns
     -------
     x, y : ndarray, ndarray
         Time and log-spaced data arrays.
     """
-    nfast = lin.shape[-1]
+    binning = _inspect_binning(binning)
+    period = _period_from_data(lin, multi)
     nslow = multi.shape[-1]
-    period = nfast // nslow
-    if nfast % nslow != 0:
-        raise ValueError("Fast and slow data are not compatible")
-    #level = int(np.log2(period))
-    #assert 2**level == period
-    #n = cfast.shape[-1] // (2**level)
-    ldata = multilevel(lin, nslow)
+
+    ldata = multilevel(lin, nslow, binning)
     xfast, cfast = merge_multilevel(ldata, mode = "full")
     xslow, cslow = merge_multilevel(multi, mode = "half")
-    t = np.concatenate((xfast, period * xslow), axis = -1)
+    t = np.concatenate((xfast, 2 * period * xslow), axis = -1)
     cc = np.concatenate((cfast, cslow), axis = -1)
     return t, cc
+
+def log_merge_count(lin_count,multi_count, binning = BINNING_MEAN):
+    """Merges multi-tau count data. This function cab be used to obtain
+    effective count data of results merged by :func:`log_merge`. You must
+    first call equivalent count function, like :func:`ccorr_multi_count`.
     
-def normalize_multi(*args, **kwargs):
+    
+    Parameters
+    ----------
+    lin_count : ndarray
+        Linear data count
+    multi_count : ndarray
+        Multilevel data count
+    binning : int
+        Binning mode used in :func:`log_merge`
+        
+    Returns
+    -------
+    x, y : ndarray, ndarray
+        Time and log-spaced count arrays.
+    """
+    period = _period_from_data(lin_count, multi_count)
+    nslow = multi_count.shape[-1]
+
+    ldata = multilevel_count(lin_count, nslow, binning = binning)
+    xfast, cfast = merge_multilevel(ldata, mode = "full")
+    xslow, cslow = merge_multilevel(multi_count, mode = "half")
+    t = np.concatenate((xfast, 2 * period * xslow), axis = -1)
+    cc = np.concatenate((cfast, cslow), axis = -1)
+    return t, cc    
+ 
+
+def t_multilevel(shape, period = 1):
+    """Returns broadcasteable time array of multilevel data with a given shape
+    """
+    shape = np.array(shape)
+    shape[1:-1] = 1 #these are for broadcasting only, so set to one
+    out = np.empty(shape = shape, dtype = int)
+    x = np.arange(shape[-1]) * period*2
+    for o in out:
+        o[...,:] = x
+        x *= 2
+    return out
+        
+def normalize_multi(data, background = None, variance = None, norm = None,  mode = "corr", 
+              scale = False, mask = None):
     """A multitau version of :func:`.core.normalize`.
     
     Performs normalization of data returned by :func:`ccorr_multi`,
@@ -1169,9 +1426,66 @@ def normalize_multi(*args, **kwargs):
     
     See documentation of :func:`.core.normalize`.
     """
-    out = kwargs.pop("out", None)
-    if out is None:
-        return tuple((normalize(d,*args[1:],**kwargs) for d in args[0]))
+
+    print1("Normalizing...")
+    lin, multi = data
+    
+    #determine what kind of data is there ('diff' or 'corr')
+    method = _method_from_data(lin)
+    scale = _inspect_scale(scale)
+    mode = _inspect_mode(mode)
+    #determine default normalization if not specified by user
+    norm = _default_norm_from_data(lin, method, norm)
+    
+    print2("   * background : {}".format(background is not None))
+    print2("   * variance   : {}".format(variance is not None))
+    print2("   * norm       : {}".format(norm))
+    print2("   * scale      : {}".format(scale))
+    print2("   * mode       : {}".format(mode))
+    print2("   * mask       : {}".format(mask is not None))
+    
+    level = disable_prints()
+    
+    
+    if (norm & NORM_WEIGHTED):
+    
+        norm_comp = (norm & NORM_SUBTRACTED)|NORM_COMPENSATED
+        norm_base = norm & NORM_SUBTRACTED        
+        
+        lin_comp = normalize(lin, background = background, variance = variance, norm = norm_comp, mode = mode,
+                  scale = scale, mask = mask)
+        multi_comp = normalize(multi, background = background, variance = variance, norm = norm_comp, mode = mode,
+                  scale = scale, mask = mask) 
+        
+        lin_base = normalize(lin, background = background, variance = variance, norm = norm_base, mode = mode,
+                  scale = scale, mask = mask)
+        multi_base = normalize(multi, background = background, variance = variance, norm = norm_base, mode = mode,
+                  scale = scale, mask = mask)   
+  
+        _scale_factor = 1. if scale == True else scale_factor(variance,mask)
+
+        #get data estimator
+        x, y = log_merge(lin_comp, multi_comp)
+        
+        x_lin = np.arange(lin_comp.shape[-1])
+        weight = weight_from_data(y, scale_factor = _scale_factor, mode = mode)
+        weight = log_interpolate(x_lin,x,weight)
+        lin = weighted_sum(lin_base,lin_comp, weight)
+        
+        period = _period_from_data(lin_comp, multi_comp)
+        x_multi = t_multilevel(multi_comp.shape, period = period)
+        weight = weight_from_data(y, scale_factor = _scale_factor, mode = mode,)
+        weight = log_interpolate(x_multi,x,weight)
+        multi = weighted_sum(multi_base,multi_comp, weight)        
+
     else:
-        return tuple((normalize(d,*args[1:],out = o, **kwargs) for d,o in zip(args[0],out)))
-        
\ No newline at end of file
+        lin = normalize(lin, background = background, variance = variance, norm = norm, mode = mode,
+                  scale = scale, mask = mask)
+        multi = normalize(multi, background = background, variance = variance, norm = norm, mode = mode,
+                  scale = scale, mask = mask)  
+    enable_prints(level)        
+    return lin, multi
+
+#if __name__ == "__main__":
+#    import doctest
+#    doctest.testmod()
\ No newline at end of file
